/**********************************************************************************/
/*                                                                                */
/*  File: progmesh.h                                                              */
/*  Author: bkenwright@xbdev.net                                                  */
/*  URL: www.xbdev.net                                                            */
/*  Date: 10-11-2005                                                              */
/*                                                                                */
/**********************************************************************************/
/*
	Progressive Mesh Algorithm (or Level Of Details LOD)

	NOTE 'Progressive', as the algorithm produces a list of values into our vertice
	array so that we can create any size of mesh we need quickly just using the
	result from our algorith

	SLOOWWW!!  At the moment, due to the 'Collapse(..)' function using lots of
	recursive loops and of course checking all the vertices each time!  This slows
	the creation code right down!  This can be optimised with a bit of thinking
	and work!  Once its created its very fast!

	The algorithm has been kept to its simplest form!  A real treat is to take it
	to the next level by optimising it and adding in some further algorithms
	to interpolate textures and other vertice values.

	The data passed to the algorithm is a list of vertices and an list of triangles
	that index our vertice list.

	To keep the triangle structure, a triangle structure is used for the 3 values
	that make up each face.

	In our algorithms we do numerous things to create our progressive mesh!  We
	create neighboring vertice lists, neighboring face lists etc, so that as our
	mesh gets smaller we can decide which vertice is worth the least amount.

	See progmesh.h for further details.

	HOW-WORKS?
	Using the single function 'BuildProgressiveMesh(..)' which takes a model in 
	the form of a list of vertices and indexed triangle face values.
	It then does the polygon reduction algorithm - reducing the model all
	the way down to 0 vertices, and then returns the order in which the
	vertices are collapsed to which neighbor each vertex is collapsed to.

	The returned map list indicates to which vertex each vertex is collapsed
	to when you render.  As you'll render the first n vertices, after n vertices
	you used the remapped points.

	GLOBALS!
	I've done no globals...so all you need to generate a progressive mesh 
	is the single function!

	MEMORY-LEAKS!
	I've tested the code over and over again and can't find any memory leaks
	but if you find any serious bugs please let me know.

*/
/**********************************************************************************/


#pragma once


#include <d3dx8.h> // D3DXVECTOR3


// Defined data type for each triangle!
class tridata 
{
  public:
	int	v[3];  // indices to vertex list
};



void BuildProgressiveMesh(	int num_verts,       // Total number of vertices in our mesh
							int num_tris,        // Total number of triangles in our mesh
							D3DXVECTOR3 * vert,  // Array of vertices for our mesh
							tridata * tri,       // Array of tris for our mesh
							int * collapse_map );// Returned filled in array of values
                                                 // which are used to render various levels
                                                 // of detail meshes.
                            // collapse_map has is the same size as num_verts, and would be
                            // defined for example as:
                            // int collapse_map = int[num_verts]; 





